// ============================================================================================
// Datei:      main.cpp
//             
// Desc:       Mainfile. Beinhaltet alles um Fenster zu erstellen + Ablaufcode
//             
// Todo:       ---
//             
// Coder:      Kongo
// Stand/Ver:  31.01.2002/1.0
// ============================================================================================


// ============================================================================================
// Libs
// ============================================================================================

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <mmsystem.h>
#include "zed4d.h"
#include "zhelp.h"
#include "vs.h"

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Variablen + Defs + Konstanten
// ============================================================================================

// Fenster

#define      WINDOW_CLASSNAME        "KVSTUT1Class"
#define      WINDOW_TITLE            "Kongos Vertex Shader Tutorial 1 Prog"

HWND         hWnd;
HINSTANCE    hInst;
bool         bMainWhile = true;
bool         bWindowActive = false;
bool         bGameOk = false;


// D3D

zD3DManager  Manager;

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Funktionen
// ============================================================================================

// Mglichkeit Device auf spezielle Fhigkeiten berprfen
bool EnumDeviceOkay(D3DCAPS8* pddDriverCaps, DWORD dw, D3DFORMAT d3dfrmt)
{
   // Kann HAL Vertex Shader? Wenn ja, dann nutzen wir ihn
   // Ansonsten verwenden wir T&L in Software
   if( (dw & D3DCREATE_HARDWARE_VERTEXPROCESSING ) ||
        (dw & D3DCREATE_MIXED_VERTEXPROCESSING ) ) {
        if( pddDriverCaps->VertexShaderVersion < D3DVS_VERSION(1,1) )
            return false;
   }
    return true;
}

// --------------------------------------------------------------------------------------------

bool Game_Init()
{
   _bttest(Manager.Init(hWnd,hInst,800,600,16,16,0,false,false,&EnumDeviceOkay),
           "Game_Init: Manager.Init failed\n");

   _bttest(VS_Init(), "Game_Init: VS_Init() failed\n");

   bGameOk = true;
   return true;
}

// --------------------------------------------------------------------------------------------

bool Game_Main()
{
   _httest(_clear(D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,D3DCOLOR_XRGB(0,0,50),1.0f,0),
           "Game_Main: clear failed\n");

   _beginscene();
   _bttest(VS_Main(), "Game_Main: VS_Main() failed\n");
   _endscene();

   _httest(_present(),
           "Game_Main: present failed\n");

   return true;
}

// --------------------------------------------------------------------------------------------

void Game_Shutdown()
{
   VS_Shutdown();
   Manager.Shutdown();
   bGameOk = false;
}

// --------------------------------------------------------------------------------------------

LRESULT CALLBACK WinProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{

   switch (msg) {

      case WM_ACTIVATEAPP:
         bWindowActive = (bool) wparam;
         return(0);
        break;

      case WM_KEYDOWN:
         if (wparam == VK_ESCAPE) {
            PostMessage(hwnd,WM_CLOSE,0,0);
            bGameOk = false;
            return(0);
         }
        break;

      case WM_SETCURSOR:
         SetCursor(NULL);
         ShowCursor(FALSE);
        break;

      case WM_DESTROY:
         bGameOk = false;
         bMainWhile = false;
         PostQuitMessage(0);
         return(0);
        break;


   }

   return(DefWindowProc(hwnd,msg,wparam,lparam));
}

// --------------------------------------------------------------------------------------------

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
   WNDCLASSEX     winclass;
   MSG            msg;
   HWND           hwnd;


   winclass.cbSize            = sizeof(WNDCLASSEX);
   winclass.style             = CS_HREDRAW | CS_VREDRAW;
   winclass.lpfnWndProc       = WinProc;
   winclass.cbClsExtra        = 0;
   winclass.cbWndExtra        = 0;
   winclass.hInstance         = hInstance;
   winclass.hIcon             = LoadIcon(NULL, IDI_APPLICATION);
   winclass.hCursor           = NULL; 
   winclass.hbrBackground     = (HBRUSH)GetStockObject(BLACK_BRUSH);
   winclass.lpszMenuName      = NULL;
   winclass.lpszClassName     = WINDOW_CLASSNAME;
   winclass.hIconSm           = LoadIcon(NULL, IDI_APPLICATION);

   if (!RegisterClassEx(&winclass))
      return -1;

   if (!(hwnd = CreateWindowEx(  WS_EX_TOPMOST,
                                 WINDOW_CLASSNAME,
                                 WINDOW_TITLE,
                                 WS_POPUP | WS_VISIBLE,
                                 0,0,
                                 GetSystemMetrics(SM_CXSCREEN),GetSystemMetrics(SM_CYSCREEN),
                                 NULL,
                                 NULL,
                                 hInstance,
                                 NULL)))
      return -2;

   hWnd = hwnd;
   hInst = hInstance;
   UpdateWindow(hwnd);

   if (!Game_Init())
      return -3;

   while (bMainWhile) {
      if (PeekMessage(&msg,hwnd,0,0,PM_REMOVE)) {
         TranslateMessage(&msg);
         DispatchMessage(&msg);
      }

      if (bWindowActive && bGameOk) {
         if (!Game_Main()) {
            bGameOk = false;
            PostMessage(hwnd,WM_CLOSE,0,0);
         }
      }
   }

   Game_Shutdown();
   return msg.wParam;
}

// --------------------------------------------------------------------------------------------
