
// ============================================================================================
// Libs
// ============================================================================================

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include "zed4d.h"
#include "zhelp.h"
#include "vs.h"
#include "vs.vsa.h"

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Vars + Defs + Konstanten
// ============================================================================================

#define NUM_VERTS 4
#define NUM_TRIS  2

// Vertex Struktur
struct vsvertex {
   float x,y,z;      // Untransformierte Position
   DWORD diffuse;    // Diffuse Farbe
   float tu,tv;      // Texturkoordinaten
};

// Vertex-Format Definition
#define D3DFVF_VSVERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE|D3DFVF_TEX1)

// 4 Vertices die ein Quadrat bilden
vsvertex vsvertices[] =
{
   //    x     y     z     diffuse     u     v 
    { -1.0f,-1.0f, 0.0f, 0xff00ff00, 1.0f, 1.0f,},// unten links
    {  1.0f,-1.0f, 0.0f, 0xff00ff00, 0.0f, 1.0f,},// unten rechts
    {  1.0f, 1.0f, 0.0f, 0xffff0000, 0.0f, 0.0f,},// oben rechts
    { -1.0f, 1.0f, 0.0f, 0xff0000ff, 1.0f, 0.0f,},// oben links
};

float  c[4] = {0.0f,0.5f,1.0f,2.0f};

// VS Deklaration
DWORD dwVSDecl[] =
{
   D3DVSD_STREAM(0),
   D3DVSD_REG(D3DVSDE_POSITION, D3DVSDT_FLOAT3 ),           //D3DVSDE_POSITION   0   
   D3DVSD_REG(D3DVSDE_DIFFUSE,  D3DVSDT_D3DCOLOR ),         //D3DVSDE_DIFFUSE    5  
   D3DVSD_REG(D3DVSDE_TEXCOORD0,D3DVSDT_FLOAT2 ),           //D3DVSDE_TEXCOORD0  7  
   D3DVSD_CONST(0,1),*(DWORD*)&c[0],*(DWORD*)&c[1],*(DWORD*)&c[2],*(DWORD*)&c[3],
   D3DVSD_END()
};

// --------------------------------------------------------------------------------------------

zD3DTextureManager TexMan;
zTexture_ID        idTexture;

#ifdef usevb
IDirect3DVertexBuffer8* pVB;
#endif

DWORD       hVertexShader;

D3DXMATRIX matProj, matView, matWorld; 

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Funktionen
// ============================================================================================

bool VS_Init()
{

#ifdef usevb
   void* v;
   pVB = NULL;
#endif

   hVertexShader = 0xffffffff;

   idTexture = TexMan.LoadFromFile("texture.bmp");

   // Projektions-Matrix
   D3DXMatrixPerspectiveFovLH( &matProj, _pi/4, 800.0f/600.0f, 1.0f, 100.0f);

   // View-Matrix
   D3DXMatrixLookAtLH( &matView, &D3DXVECTOR3(0.0f,1.0f,-4.0f),  // From
                                 &D3DXVECTOR3(0.0f,0.0f,0.0f),   // At
                                 &D3DXVECTOR3(0.0f,1.0f,0.0f));  // Up

/*
   EM.Write("Input for View: From(0.0f,1.0f,-4.0f) At(0.0f,0.0f,0.0f) Up(0.0f,1.0f,0.0f)\n");
   EM.WriteFormat("Input for Proj: FOV: %g, Aspect: %g, ZN: 1.0f, ZF: 100.0f\n",(_pi/4),(800.0f/600.0f));
                                                     
   EM.WriteFormat("\nView-Matrix:\n%g %g %g %g\n%g %g %g %g\n%g %g %g %g\n%g %g %g %g\n",
      matView._11,matView._12,matView._13,matView._14,
      matView._21,matView._22,matView._23,matView._24,
      matView._31,matView._32,matView._33,matView._44,
      matView._41,matView._42,matView._43,matView._44);
   EM.WriteFormat("\nProj-Matrix:\n%g %g %g %g\n%g %g %g %g\n%g %g %g %g\n%g %g %g %g\n",
      matProj._11,matProj._12,matProj._13,matProj._14,
      matProj._21,matProj._22,matProj._23,matProj._24,
      matProj._31,matProj._32,matProj._33,matProj._44,
      matProj._41,matProj._42,matProj._43,matProj._44);

   D3DXMATRIX trans,matWVP;
   float time = timeGetTime()*0.001f;

   EM.WriteFormat("Input for Rotation: %g\n",time);

   // Objekt um Y-Achse rotieren
   D3DXMatrixRotationY( &matWorld, time);
   // Transpose der matWorldViewProj
   matWVP = matWorld*matView*matProj;
   D3DXMatrixTranspose( &trans, &matWVP);

   EM.WriteFormat("\nWorld-Matrix:\n%g %g %g %g\n%g %g %g %g\n%g %g %g %g\n%g %g %g %g\n",
      matWorld._11,matWorld._12,matWorld._13,matWorld._14,
      matWorld._21,matWorld._22,matWorld._23,matWorld._24,
      matWorld._31,matWorld._32,matWorld._33,matWorld._44,
      matWorld._41,matWorld._42,matWorld._43,matWorld._44);

   EM.WriteFormat("\nmatWVP-Matrix:\n%g %g %g %g\n%g %g %g %g\n%g %g %g %g\n%g %g %g %g\n",
      matWVP._11,matWVP._12,matWVP._13,matWVP._14,
      matWVP._21,matWVP._22,matWVP._23,matWVP._24,
      matWVP._31,matWVP._32,matWVP._33,matWVP._44,
      matWVP._41,matWVP._42,matWVP._43,matWVP._44);

   EM.WriteFormat("\nmatTrans-Matrix:\n%g %g %g %g\n%g %g %g %g\n%g %g %g %g\n%g %g %g %g\n",
      trans._11,trans._12,trans._13,trans._14,
      trans._21,trans._22,trans._23,trans._24,
      trans._31,trans._32,trans._33,trans._44,
      trans._41,trans._42,trans._43,trans._44);
*/

   // Kein Culling, dadurch sehen wir auch die Rckseite des Quadrats
   _setrs(D3DRS_CULLMODE, D3DCULL_NONE);

#ifdef usevb
   // Vertexbuffer
   _httest(pDev->CreateVertexBuffer(NUM_VERTS*sizeof(vsvertex), D3DUSAGE_WRITEONLY,
           D3DFVF_VSVERTEX,D3DPOOL_DEFAULT, &pVB), "VS_Init: dev->CreateVB failed\n");

   _httest(pVB->Lock(0, 0, (BYTE**)&v, 0),
           "VS_Init: vb->lock failed\n");

   memcpy(v, vsvertices, NUM_VERTS*sizeof(vsvertex));

   _httest(pVB->Unlock(),
           "VS_Init: vb->unlock failed\n");
#endif

   // Create Vertex Shader
   _httest(pDev->CreateVertexShader(dwVSDecl, dwVsVertexShader, &hVertexShader, 0),
           "VS_Init: dev->CreateVS failed\n");

   return true;
}

// --------------------------------------------------------------------------------------------

bool VS_Main()
{
   D3DXMATRIX trans,matWVP;
   float color[4] = {0,1,0,0};

   // Objekt um Y-Achse rotieren
   D3DXMatrixRotationY( &matWorld, timeGetTime()*0.001f);
   // Transpose der matWorldViewProj
   matWVP = matWorld*matView*matProj;
   D3DXMatrixTranspose( &trans, &matWVP);

   //  Konstanten fr den Vertex Shader setzen
	_httest(pDev->SetVertexShaderConstant( 4 , &trans , 4 ),
           "VS_Main: dev->SetVSC failed\n");

   _httest(pDev->SetVertexShaderConstant(8, color, 1),
           "VS_Main: dev->SetVSC failed\n");

   // Textur an Stage 0 binden
   TexMan.BindTexture(idTexture);

#ifdef usevb
   // Vertexbuffer zu Stream 0 mappen
   _httest(_setss(0,pVB,sizeof(vsvertex)),
           "VS_Main: setss failed\n");
#endif

   // Vertex Shader setzen
   _httest(_setvs(hVertexShader),
           "VS_Main: setvs failed\n");

   // Rendern des Quadrats
#ifdef usevb
   _httest(_drawp(D3DPT_TRIANGLEFAN,0,NUM_TRIS),
           "VS_Main: drawp failed\n");
#else
   _drawpu(D3DPT_TRIANGLEFAN,NUM_TRIS,vsvertices,sizeof(vsvertex));
#endif

   return true;
}

// --------------------------------------------------------------------------------------------

void VS_Shutdown()
{
   // Wenn ein Vertex Shader erzeugt wurde, diesen auch wieder frei geben
   if (hVertexShader != 0xffffffff) {
      pDev->DeleteVertexShader(hVertexShader);
      hVertexShader = 0xffffffff;
   }

   // Textur freigeben
   TexMan.UnbindTexture();
   TexMan.ReleaseTextureHandles();

#ifdef usevb
   _release(pVB);
#endif

}

// --------------------------------------------------------------------------------------------

